;******************************************************************************
;                                                                             *
;    Filename: LegCond.asm                                                    *
;    Date: 30/06/2004                                                         *
;    File Version: 1.2                                                        *
;                                                                             *
;    Author: Virgilio Mattoli                                                 *
;    Company: BDML                                                            *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files required:         P18F1320.INC                                     *
;			     18F1X20_conf.h				      *	
;                            18fmacro.asm				      *
;                            18f_perf.asm				      *
;                            Loop18F16Bit.asm                                 *
;                            18F_InternalAD.asm                               *
;                                                                             *
;******************************************************************************

	LIST P=18F1320		;directive to define processor
	#include <P18F1320.INC>	;processor specific variable definitions

	#define FREQUENCY D'8000'	; Frequenza del procesore in KHz (va considerato anche l'eventuale PLL)

	#include "18F1X20_conf.h"

	CBLOCK	0x000
	SER_TEMP, COUNT_H, COUNT_L, COUNT_TMP, TEMP_VIS, TEMP_VIS2

	DAC_BUFF_H, DAC_BUFF_L

	TEMP_LOOP, TEMP_LOOP_H, TEMP_THR_L, TEMP_THR_H, STATUS_LOOP, IN_REG_L, IN_REG_H
	OUT_REG_L, OUT_REG_H, THRESHOLD_L, THRESHOLD_H, DELTA_MAX_L, DELTA_MAX_H

	TEMP_adc, AN_H, AN_L, CHANNEL
	ADC_ON
	
	KEY, TEMP_KEY, TEMP_KEY2, TEMP_KEY3
	ENDC


#DEFINE PORTLED PORTA
#DEFINE TRISLED TRISA
#DEFINE B_BLED	7
#DEFINE BLED	PORTLED, B_BLED 

#DEFINE B_RLED	6
#DEFINE RLED	PORTLED, B_RLED 

#DEFINE PORTDAC PORTB
#DEFINE TRISDAC TRISB
#DEFINE B_CS_G	3
#DEFINE B_CS_A	7
#DEFINE B_OUT	6
#DEFINE B_CLK	5

#DEFINE CS_G	PORTDAC, B_CS_G
#DEFINE CS_A	PORTDAC, B_CS_A
#DEFINE OUT	PORTDAC, B_OUT
#DEFINE CLK	PORTDAC, B_CLK

;******************************************************************************

		ORG	0x0000

		goto	Main		;go to start of main code

;******************************************************************************
;High priority interrupt vector

		ORG	0x0008

		bra	HighInt		;go to high priority interrupt routine


;******************************************************************************
;High priority interrupt routine

HighInt:
;	*** high priority interrupt code goes here ***
;	BTFSC	PIR1, CCP1IF		; INTERRUPT SU COMPARE
;	CALL	TIMER_UPDATE		;
					;
	BTFSC	PIR1, RCIF		; INTERRUPT SU RICEZIONE BYTE RS232
	CALL	DECODE			;
					;
;	BTFSC	PIRx, YYY		;
;	CALL	ZZZZZ			;

		retfie	FAST

;******************************************************************************

	#include "18fmacro.asm"
	#include "18f_perf.asm"
	#include "18_Math.asm"
	#include "Loop18F16Bit.asm"
	#include "18F_InternalAD.asm"
	#include "Keys18F.asm"

Main:

SetPort:
	CLRF	INTCON		;

	MOVLW	B'01110011'	;	
	MOVWF	OSCCON		;

	CALL	INIT_ADC_INT	;
	BCF	INTCON2, 7	; PULL-UP ENABLED BY LATCH VALUE
	
	BCF	TRISLED, B_BLED	;
	BCF	BLED		;
	BCF	TRISLED, B_RLED	;
	BCF	RLED		;
				;
	CALL	SETUP_DAC	;
				;
	DEV_SETTING		;
	CALL	INIT_KEYS	;
	
	BSF	PORTB, 0	; ENABLE INDIVIDUAL PULL-UP
	BSF	PORTB, 2	;
	BSF	PORTB, 4	;



Init:
	BSF	BLED		;
	MILLISECONDI	D'200'	;
	BCF	BLED		;
	MILLISECONDI	D'200'	;
	BSF	BLED		;
	MILLISECONDI	D'200'	;
	BCF	BLED		;
	MILLISECONDI	D'200'	;
	BSF	BLED		;

	CALL	INIT_LOOP	;
;	BSF	INVERS_LOOP	; INVERSIONE DI POLARITA' PER IL LOOP CHIUSO

	BSF	INTCON, GIE	;
	CLRF	ADC_ON		;

MainLoop:
	CALL	WAITFORKEY	;

	BTFSC	ADC_ON, 0	;
	CALL	SEND_STREAM	;	

	GOTO	MainLoop	;


SEND_STREAM:			;
	MOVLW	0x00		;  - STRAIGHT GAUGE
	MOVWF	CHANNEL		;
	CALL	ACQUIRE		;
	MOVF	AN_H, W		;
	CALL	SER_VIS_BIN	;
	MOVF	AN_L, W		;
	CALL	SER_VIS_BIN	;
		
	MOVLW	D'9'		;
	CALL	RS232PUTCHAR	;

	MOVLW	0x01		;  - CURVE GAUGE
	MOVWF	CHANNEL		;
	CALL	ACQUIRE		;
	MOVF	AN_H, W		;
	CALL	SER_VIS_BIN	;
	MOVF	AN_L, W		;
	CALL	SER_VIS_BIN	;

	MOVLW	D'9'		;
	CALL	RS232PUTCHAR	;

	MOVLW	0x02		; - ACC X
	MOVWF	CHANNEL		;
	CALL	ACQUIRE		;
	MOVF	AN_H, W		;
	CALL	SER_VIS_BIN	;
	MOVF	AN_L, W		;
	CALL	SER_VIS_BIN	;

	MOVLW	D'9'		;
	CALL	RS232PUTCHAR	;

	MOVLW	0x03		; - ACC Y
	MOVWF	CHANNEL		;
	CALL	ACQUIRE		;
	MOVF	AN_H, W		;
	CALL	SER_VIS_BIN	;
	MOVF	AN_L, W		;
	CALL	SER_VIS_BIN	;

	MOVLW	D'13'		;
	CALL	RS232PUTCHAR	;
	MOVLW	D'10'		;
	CALL	RS232PUTCHAR	;
;	MILLISECONDI	D'100'	;
	RETURN			;


KEY_0_Routine:				; ACCELEROMETERS
	MOVLW	D'10'			;
	MOVWF	DELTA_MAX_L		;
	CLRF	DELTA_MAX_H		;
	BCF	BLED			;
	CALL	AUTO_ZERO_ACC_X		;
	CALL	AUTO_ZERO_ACC_Y		;
	BSF	BLED			;
	RETURN				;


KEY_2_Routine:				; GAUGES
	MOVLW	D'100'			;
	MOVWF	DELTA_MAX_L		;
	CLRF	DELTA_MAX_H		;
	BCF	BLED			;
	CALL	AUTO_ZERO_GAUGE_A	;
	CALL	AUTO_ZERO_GAUGE_B	;
	BSF	BLED			;

	RETURN				;


;****************************************************************************
;
;	CONVERTE LA CIFRA IN ASCII CORRISPONDENTE
;
;****************************************************************************

AUTO_ZERO_GAUGE_A			;
	MOVLW	0x00			;
	MOVWF	OUT_REG_L		;
	MOVLW	0x08			;
	MOVWF	OUT_REG_H		;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_G			;
	CALL	DAC_ChA_UPD		; INIT DAC (A - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_G			;
AUTO_ZERO_GAUGE_LOOP_A			;
	MILLISECONDI	D'10'		;
	MOVLW	0x00			; SELECTION: CHANNEL 0 (STRAIGHT GAUGE )
	MOVWF	CHANNEL			;  
	CALL	ACQUIRE			; AND ACQUISITION
	MOVFF	AN_L, IN_REG_L		;
	MOVFF	AN_H, IN_REG_H		; UPDATE CLOSED LOOP INPUT REGISTERS
					;
	BSF	INVERS_LOOP		; INVERSIONE DI POLARITA' PER IL LOOP CHIUSO
	CALL	LOOP_UPDATE		;;; CLOSE LOOP ROUTINE (SINGLE STEP) 
					;
	MOVLW	B'11110000'		;; TEST X OVERFLOW 16 BIT
	ANDWF	OUT_REG_H, W		;;
	BZ	NO_OVERFLOW_LOOP_A	;;
	SETF	OUT_REG_L		;; IF OVERFLOW -->  OUT_REG == 0x0FFF
	SETF	OUT_REG_H		;; 12 BIT SATURATION
	MOVLW	B'00001111'		;;
	ANDWF	OUT_REG_H, F		;;
	NO_OVERFLOW_LOOP_A		;;
					;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_G			;
	CALL	DAC_ChA_UPD		; SETTING DAC (A - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_G			;
	BTFSS	LOCKED			;; VALUE ACHIEVED!
	BRA	AUTO_ZERO_GAUGE_LOOP_A	;;
	RETURN				;

AUTO_ZERO_GAUGE_B			;
	MOVLW	0x00			;
	MOVWF	OUT_REG_L		;
	MOVLW	0x08			;
	MOVWF	OUT_REG_H		;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_G			;
	CALL	DAC_ChB_UPD		; INIT DAC (B - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_G			;
AUTO_ZERO_GAUGE_LOOP_B			;
	MILLISECONDI	D'10'		;
	MOVLW	0x01			; SELECTION: CHANNEL 1 (CURVE GAUGE)
	MOVWF	CHANNEL			;  
	CALL	ACQUIRE			; AND ACQUISITION
	MOVFF	AN_L, IN_REG_L		;
	MOVFF	AN_H, IN_REG_H		; UPDATE CLOSED LOOP INPUT REGISTERS
					;
	BSF	INVERS_LOOP		; INVERSIONE DI POLARITA' PER IL LOOP CHIUSO
	CALL	LOOP_UPDATE		;;; CLOSE LOOP ROUTINE (SINGLE STEP) 
					;
	MOVLW	B'11110000'		;; TEST X OVERFLOW 16 BIT
	ANDWF	OUT_REG_H, W		;;
	BZ	NO_OVERFLOW_LOOP_B	;;
	SETF	OUT_REG_L		;; IF OVERFLOW -->  OUT_REG == 0x0FFF
	SETF	OUT_REG_H		;; 12 BIT SATURATION
	MOVLW	B'00001111'		;;
	ANDWF	OUT_REG_H, F		;;
	NO_OVERFLOW_LOOP_B		;;
					;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_G			;
	CALL	DAC_ChB_UPD		; SETTING DAC (B - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_G			;
	BTFSS	LOCKED			;; VALUE ACHIEVED!
	BRA	AUTO_ZERO_GAUGE_LOOP_B	;;
					;
	BSF	BLED			;
	RETURN


AUTO_ZERO_ACC_X				;
	MOVLW	0x00			;
	MOVWF	OUT_REG_L		;
	MOVLW	0x08			;
	MOVWF	OUT_REG_H		;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_A			;
	CALL	DAC_ChA_UPD		; INIT DAC (A - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_A			;
AUTO_ZERO_ACC_LOOP_X			;
	MILLISECONDI	D'10'		;
	MOVLW	0x02			; SELECTION: CHANNEL 2 (ACC X)
	MOVWF	CHANNEL			;  
	CALL	ACQUIRE			; AND ACQUISITION
	MOVFF	AN_L, IN_REG_L		;
	MOVFF	AN_H, IN_REG_H		; UPDATE CLOSED LOOP INPUT REGISTERS
					;
	BSF	INVERS_LOOP		; INVERSIONE DI POLARITA' PER IL LOOP CHIUSO
	CALL	LOOP_UPDATE		;;; CLOSE LOOP ROUTINE (SINGLE STEP) 
					;
	MOVLW	B'11110000'		;; TEST X OVERFLOW 16 BIT
	ANDWF	OUT_REG_H, W		;;
	BZ	NO_OVERFLOW_LOOP_X	;;
	SETF	OUT_REG_L		;; IF OVERFLOW -->  OUT_REG == 0x0FFF
	SETF	OUT_REG_H		;; 12 BIT SATURATION
	MOVLW	B'00001111'		;;
	ANDWF	OUT_REG_H, F		;;
	NO_OVERFLOW_LOOP_X		;;
					;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_A			;
	CALL	DAC_ChA_UPD		; SETTING DAC (A - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_A			;
	BTFSS	LOCKED			;; VALUE ACHIEVED!
	BRA	AUTO_ZERO_ACC_LOOP_X	;;
	RETURN				;

AUTO_ZERO_ACC_Y				;
	MOVLW	0x00			;
	MOVWF	OUT_REG_L		;
	MOVLW	0x08			;
	MOVWF	OUT_REG_H		;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_A			;
	CALL	DAC_ChB_UPD		; INIT DAC (B - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_A			;
AUTO_ZERO_ACC_LOOP_Y			;
	MILLISECONDI	D'10'		;
	MOVLW	0x03			; SELECTION: CHANNEL 3 (ACC Y)
	MOVWF	CHANNEL			;  
	CALL	ACQUIRE			; AND ACQUISITION
	MOVFF	AN_L, IN_REG_L		;
	MOVFF	AN_H, IN_REG_H		; UPDATE CLOSED LOOP INPUT REGISTERS
					;
	BSF	INVERS_LOOP		; INVERSIONE DI POLARITA' PER IL LOOP CHIUSO
	CALL	LOOP_UPDATE		;;; CLOSE LOOP ROUTINE (SINGLE STEP) 
					;
	MOVLW	B'11110000'		;; TEST X OVERFLOW 16 BIT
	ANDWF	OUT_REG_H, W		;;
	BZ	NO_OVERFLOW_LOOP_Y	;;
	SETF	OUT_REG_L		;; IF OVERFLOW -->  OUT_REG == 0x0FFF
	SETF	OUT_REG_H		;; 12 BIT SATURATION
	MOVLW	B'00001111'		;;
	ANDWF	OUT_REG_H, F		;;
	NO_OVERFLOW_LOOP_Y		;;
					;
	MOVFF	OUT_REG_L, DAC_BUFF_L	;
	MOVFF	OUT_REG_H, DAC_BUFF_H	;
	BCF	CS_A			;
	CALL	DAC_ChB_UPD		; SETTING DAC (B - CHANNEL) FOR OFFSET COMPENSATION
	BSF	CS_A			;
	BTFSS	LOCKED			;; VALUE ACHIEVED!
	BRA	AUTO_ZERO_ACC_LOOP_Y	;;
					;
	BSF	BLED			;
	RETURN



;****************************************************************************
;
;	CONVERTE LA CIFRA IN ASCII CORRISPONDENTE
;
;****************************************************************************
DIGIT
		MOVWF		TEMP_VIS		;
		TabSetting 	DIGIT_TAB		;
		MOVF		TEMP_VIS, W		;
		ANDLW		B'00001111'		;
		AddTab					;
		TBLRD*					;
		MOVF		TABLAT, W		;
		RETURN					;
DIGIT_TAB
		DB		"0123456789ABCDEF"	;


;****************************************************************************
;
;	2 DIGIT VISUALIZATION IN EXE (1 BYTE) ON SERIAL RS232
;
;****************************************************************************
SER_VIS_BIN
	MOVWF	TEMP_VIS2		;
	SWAPF	TEMP_VIS2, W		;
	ANDLW	0x0F			;
	CALL	DIGIT			;
	CALL	RS232PUTCHAR		;
	MOVF	TEMP_VIS2, W		;
	ANDLW	0x0F			;
	CALL	DIGIT			;
	CALL	RS232PUTCHAR		;
	RETURN				;

RS232PUTCHAR
TXS	BTFSS	PIR1, TXIF		;
	BRA	TXS			;
	MOVWF	TXREG			;
	RETURN				;


;****************************************************************************
;
;	INSTRUCTION DECODE
;
;****************************************************************************

DECODE:
        MOVFF	RCREG, SER_TEMP        	;
;	MOVFF	SER_TEMP, TXREG		; ECHO	

	BCF	STATUS, Z	;
	MOVLW	"0"		; TEST DAC_0_A - STRAIGHT GAUGE
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	TEST_DAC_0_A	; 

	BCF	STATUS, Z	;
	MOVLW	"1"		; TEST DAC_0_B - CURVE GAUGE
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	TEST_DAC_0_B	; 

	BCF	STATUS, Z	;
	MOVLW	"2"		; TEST DAC_1_A - ACC X
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	TEST_DAC_1_A	; 

	BCF	STATUS, Z	;
	MOVLW	"3"		; TEST DAC_1_B - ACC Y
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	TEST_DAC_1_B	; 

	BCF	STATUS, Z	;
	MOVLW	"9"		; STREAM ON
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	STREAM_ON	; 

	BCF	STATUS, Z	;
	MOVLW	"8"		; STREAM OFF
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	STREAM_OFF	; 
;---

	BCF	STATUS, Z	;
	MOVLW	"5"		; RESET_GAUGES
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	KEY_2_Routine	; 

	BCF	STATUS, Z	;
	MOVLW	"6"		; RESET_ACC
	SUBWF	SER_TEMP, W	;
	BTFSC	STATUS, Z	;
	CALL	KEY_0_Routine	; 


;	BCF	STATUS, Z	;
;	MOVLW	.50		; ...
;	SUBWF	SER_TEMP, W	;
;	BTFSC	STATUS, Z	;
;	CALL	....		; 
	RETURN

STREAM_ON
	SETF	ADC_ON
	RETURN

STREAM_OFF
	CLRF	ADC_ON
	RETURN


TEST_DAC_0_A: 
	MOVLW	0x08
	MOVWF	DAC_BUFF_H
	MOVLW	0x00
	MOVWF	DAC_BUFF_L
	BCF	CS_G
	CALL	DAC_ChA_UPD
	BSF	CS_G
	RETURN

TEST_DAC_0_B: 
	MOVLW	0x08
	MOVWF	DAC_BUFF_H
	MOVLW	0x00
	MOVWF	DAC_BUFF_L
	BCF	CS_G
	CALL	DAC_ChB_UPD
	BSF	CS_G
	RETURN

TEST_DAC_1_A: 
	MOVLW	0x08
	MOVWF	DAC_BUFF_H
	MOVLW	0x00
	MOVWF	DAC_BUFF_L
	BCF	CS_A
	CALL	DAC_ChA_UPD
	BSF	CS_A
	RETURN

TEST_DAC_1_B: 
	MOVLW	0x08
	MOVWF	DAC_BUFF_H
	MOVLW	0x00
	MOVWF	DAC_BUFF_L
	BCF	CS_A
	CALL	DAC_ChB_UPD
	BSF	CS_A
	RETURN

;****************************************************************************
;
;	DAC MAX5154 INTERFACING ROUTINES
;
;****************************************************************************
;
; Use appropriate CS!
;

SETUP_DAC:
	BCF	TRISDAC, B_CS_G	;
	BCF	TRISDAC, B_CS_A	;
	BCF	TRISDAC, B_CLK	;
	BCF	TRISDAC, B_OUT 	;
	BSF	CS_G		;
	BSF	CS_A		;
	BCF	CLK		;
	BCF	OUT		;
	CALL	DAC_RESET	;
	RETURN			;

DAC_RESET:
	BCF	OUT		; Send 0
	BSF	CLK		;
	BCF	CLK		;
				;
	BSF	OUT		; Send 1
	BSF	CLK		;
	BCF	CLK		;
				;
	BSF	OUT		; Send 1
	BSF	CLK		;
	BCF	CLK		;
				;
	BCF	OUT		;
	MOVLW	D'13'		; Send All Other 0
RES_LOOP			;
	BSF	CLK		;
	BCF	CLK		;
	DECFSZ	WREG		;
	BRA	RES_LOOP	;
				;
	RETURN			;


DAC_ChA_UPD:
	BCF	OUT		; Send 0
	BSF	CLK		;
	BCF	CLK		;
				;
	BSF	OUT		; Send 1
	BSF	CLK		;
	BCF	CLK		;
				;
	BCF	OUT		; Send 0
	BSF	CLK		;
	BCF	CLK		;
				;
	MOVLW	D'12'		;
A_LOOP	BTFSS	DAC_BUFF_H, 3	;
	BCF	OUT		;
	BTFSC	DAC_BUFF_H, 3	;
	BSF	OUT		;
				;
	BSF	CLK		;
	RLCF	DAC_BUFF_L, F	;
	RLCF	DAC_BUFF_H, F	;
	BCF	CLK		;
				;
	DECFSZ	WREG		;
	BRA	A_LOOP		;
				;
	BCF	OUT		; Send 0
	BSF	CLK		;
	BCF	CLK		;
	RETURN

DAC_ChB_UPD:
	BSF	OUT		; Send 1
	BSF	CLK		;
	BCF	CLK		;
				;
	BSF	OUT		; Send 1
	BSF	CLK		;
	BCF	CLK		;
				;
	BCF	OUT		; Send 0
	BSF	CLK		;
	BCF	CLK		;
				;
	MOVLW	D'12'		;
B_LOOP	BTFSS	DAC_BUFF_H, 3	;
	BCF	OUT		;
	BTFSC	DAC_BUFF_H, 3	;
	BSF	OUT		;
				;
	BSF	CLK		;
	RLCF	DAC_BUFF_L, F	;
	RLCF	DAC_BUFF_H, F	;
	BCF	CLK		;
				;
	DECFSZ	WREG		;
	BRA	B_LOOP		;
				;
	BCF	OUT		; Send 0
	BSF	CLK		;
	BCF	CLK		;
	RETURN			;

;******************************************************************************
;End of program

		END
